//
// Dirty test of BLOB support in OGR SQLite driver
//
// The write.cpp program reads sample BLOB data from multipoint.wkb file,
// and assigns it as a value of feature attribute, in SQLite database.
// Next, read.cpp progam fetches the only written feature and tests BLOB data.
//

#include "ogrsf_frmts.h"
#include <cassert>
#include <cstdio>
#include <fstream>
using namespace std;

int main()
{
    const char *pszDriverName = "SQLite";
    const char *pszDatabase = "test.sdb";

    OGRRegisterAll();

    /////////////////////////////////////////////////////////////////////////
    // Find driver

    OGRSFDriver *poDriver = NULL;
    poDriver = OGRSFDriverRegistrar::GetRegistrar()->GetDriverByName( pszDriverName );
    if( poDriver == NULL )
    {
        printf( "%s driver not available.\n", pszDriverName );
        exit( 1 );
    }

    /////////////////////////////////////////////////////////////////////////
    // Create SQLite database

    OGRDataSource *poDS = NULL;
    poDS = poDriver->CreateDataSource( pszDatabase, NULL );
    if( poDS == NULL )
    {
        printf( "Creation of database failed.\n Remove the '%s' file if it exists. \n", pszDatabase );
        exit( 1 );
    }

    /////////////////////////////////////////////////////////////////////////
    // Create table with BLOB field

    OGRLayer *poLayer = NULL;
    poLayer = poDS->CreateLayer( "t1", NULL, wkbPoint, NULL );
    if( poLayer == NULL )
    {
        printf( "Layer creation failed.\n" );
        exit( 1 );
    }

    OGRFieldDefn oField( "BINDATA", OFTBinary );

    if( poLayer->CreateField( &oField ) != OGRERR_NONE )
    {
        printf( "Creating Name field failed.\n" );
        exit( 1 );
    }

    /////////////////////////////////////////////////////////////////////////
    // Create new feature

    OGRFeature *poFeature = NULL;
    poFeature = OGRFeature::CreateFeature( poLayer->GetLayerDefn() );
    assert(NULL != poFeature);
    
    /////////////////////////////////////////////////////////////////////////
    // Read sample BLOB data from file - geometry encoded in WKB format

    std::ifstream fs("multipoint.wkb", ios::in|ios::binary|ios::ate);
    if (!fs.is_open())
    {
        printf( "Can not open multipoint.wkb file\n" );
        exit( 1 );
    }

    std::ifstream::pos_type size = fs.tellg();
    char* buf = new char[size];
    fs.seekg(0, ios::beg);
    fs.read(buf, size);
    fs.close();

    /////////////////////////////////////////////////////////////////////////
    // Set BLOB value to the feature attribute

    int fdx = poFeature->GetFieldIndex("BINDATA");
    poFeature->SetField( fdx, size, reinterpret_cast<GByte*>(buf) );

    /////////////////////////////////////////////////////////////////////////
    // Assign geometry

    OGRPoint pt;
    pt.setX( 50.1234 );
    pt.setY( 20.1234 );
    poFeature->SetGeometry( &pt ); 

    /////////////////////////////////////////////////////////////////////////
    // Submit the feature

    if( poLayer->CreateFeature( poFeature ) != OGRERR_NONE )
    {
        printf( "Failed to create feature in shapefile.\n" );
        exit( 1 );
    }

    OGRFeature::DestroyFeature( poFeature );
    OGRDataSource::DestroyDataSource( poDS );
}
